@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -537,6 +537,7 @@ class CUserGroup extends CApiService {
 
 		foreach ($usrgrps as $usrgrp) {
 			if (self::userGroupDisabled($usrgrp, $method, $db_usrgrps)
+					&& array_key_exists('userids', $usrgrp)
 					&& uint_in_array(self::$userData['userid'], $usrgrp['userids'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 					_('User cannot add himself to a disabled group or a group with disabled GUI access.')
@@ -604,17 +605,17 @@ class CUserGroup extends CApiService {
 		}
 
 		$db_users = DBselect(
-			'SELECT u.userid,u.alias,count(ug.usrgrpid) as usrgrp_num'.
+			'SELECT u.userid,u.username,count(ug.usrgrpid) as usrgrp_num'.
 			' FROM users u,users_groups ug'.
 			' WHERE u.userid=ug.userid'.
 				' AND '.dbConditionInt('u.userid', array_keys($del_userids)).
-			' GROUP BY u.userid,u.alias'
+			' GROUP BY u.userid,u.username'
 		);
 
 		while ($db_user = DBfetch($db_users)) {
 			if ($db_user['usrgrp_num'] == $del_userids[$db_user['userid']]) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('User "%1$s" cannot be without user group.', $db_user['alias'])
+					_s('User "%1$s" cannot be without user group.', $db_user['username'])
 				);
 			}
 		}
@@ -926,6 +927,21 @@ class CUserGroup extends CApiService {
 			);
 		}
 
+		// Check if user groups are used in scheduled reports.
+		$db_reports = DBselect(
+			'SELECT r.name,rug.usrgrpid'.
+			' FROM report r,report_usrgrp rug'.
+			' WHERE r.reportid=rug.reportid'.
+				' AND '.dbConditionInt('rug.usrgrpid', $usrgrpids),
+			1
+		);
+
+		if ($db_report = DBfetch($db_reports)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('User group "%1$s" is report "%2$s" recipient.',
+				$db_usrgrps[$db_report['usrgrpid']]['name'], $db_report['name']
+			));
+		}
+
 		$this->checkUsersWithoutGroups($usrgrps);
 	}
 
@@ -934,57 +950,65 @@ class CUserGroup extends CApiService {
 
 		// adding users
 		if ($options['selectUsers'] !== null && $options['selectUsers'] != API_OUTPUT_COUNT) {
+			$dbUsers = [];
 			$relationMap = $this->createRelationMap($result, 'usrgrpid', 'userid', 'users_groups');
+			$related_ids = $relationMap->getRelatedIds();
 
-			$get_access = ($this->outputIsRequested('gui_access', $options['selectUsers'])
-				|| $this->outputIsRequested('debug_mode', $options['selectUsers'])
-				|| $this->outputIsRequested('users_status', $options['selectUsers'])) ? true : null;
+			if ($related_ids) {
+				$get_access = ($this->outputIsRequested('gui_access', $options['selectUsers'])
+					|| $this->outputIsRequested('debug_mode', $options['selectUsers'])
+					|| $this->outputIsRequested('users_status', $options['selectUsers'])) ? true : null;
 
-			$dbUsers = API::User()->get([
-				'output' => $options['selectUsers'],
-				'userids' => $relationMap->getRelatedIds(),
-				'getAccess' => $get_access,
-				'preservekeys' => true
-			]);
+				$dbUsers = API::User()->get([
+					'output' => $options['selectUsers'],
+					'userids' => $related_ids,
+					'getAccess' => $get_access,
+					'preservekeys' => true
+				]);
+			}
 
 			$result = $relationMap->mapMany($result, $dbUsers, 'users');
 		}
 
 		// adding usergroup rights
 		if ($options['selectRights'] !== null && $options['selectRights'] != API_OUTPUT_COUNT) {
+			$db_rights = [];
 			$relationMap = $this->createRelationMap($result, 'groupid', 'rightid', 'rights');
+			$related_ids = $relationMap->getRelatedIds();
 
-			if (is_array($options['selectRights'])) {
-				$pk_field = $this->pk('rights');
+			if ($related_ids) {
+				if (is_array($options['selectRights'])) {
+					$pk_field = $this->pk('rights');
 
-				$output_fields = [
-					$pk_field => $this->fieldId($pk_field, 'r')
-				];
+					$output_fields = [
+						$pk_field => $this->fieldId($pk_field, 'r')
+					];
 
-				foreach ($options['selectRights'] as $field) {
-					if ($this->hasField($field, 'rights')) {
-						$output_fields[$field] = $this->fieldId($field, 'r');
+					foreach ($options['selectRights'] as $field) {
+						if ($this->hasField($field, 'rights')) {
+							$output_fields[$field] = $this->fieldId($field, 'r');
+						}
 					}
+
+					$output_fields = implode(',', $output_fields);
+				}
+				else {
+					$output_fields = 'r.*';
 				}
 
-				$output_fields = implode(',', $output_fields);
-			}
-			else {
-				$output_fields = 'r.*';
-			}
+				$db_rights = DBfetchArray(DBselect(
+					'SELECT '.$output_fields.
+					' FROM rights r'.
+					' WHERE '.dbConditionInt('r.rightid', $related_ids).
+						((self::$userData['type'] == USER_TYPE_SUPER_ADMIN) ? '' : ' AND r.permission>'.PERM_DENY)
+				));
+				$db_rights = zbx_toHash($db_rights, 'rightid');
 
-			$db_rights = DBfetchArray(DBselect(
-				'SELECT '.$output_fields.
-				' FROM rights r'.
-				' WHERE '.dbConditionInt('r.rightid', $relationMap->getRelatedIds()).
-					((self::$userData['type'] == USER_TYPE_SUPER_ADMIN) ? '' : ' AND r.permission>'.PERM_DENY)
-			));
-			$db_rights = zbx_toHash($db_rights, 'rightid');
-
-			foreach ($db_rights as &$db_right) {
-				unset($db_right['rightid'], $db_right['groupid']);
+				foreach ($db_rights as &$db_right) {
+					unset($db_right['rightid'], $db_right['groupid']);
+				}
+				unset($db_right);
 			}
-			unset($db_right);
 
 			$result = $relationMap->mapMany($result, $db_rights, 'rights');
 		}
