@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,19 +19,7 @@
 
 
 jQuery(function($) {
-	var ZBX_STYLE_CLASS = 'multiselect-control',
-		KEY = {
-			ARROW_DOWN: 40,
-			ARROW_LEFT: 37,
-			ARROW_RIGHT: 39,
-			ARROW_UP: 38,
-			BACKSPACE: 8,
-			DELETE: 46,
-			ENTER: 13,
-			ESCAPE: 27,
-			TAB: 9
-		};
-
+	var ZBX_STYLE_CLASS = 'multiselect-control';
 	const MS_ACTION_POPUP = 0;
 	const MS_ACTION_AUTOSUGGEST = 1;
 
@@ -387,10 +375,15 @@ jQuery(function($) {
 			}
 
 			$obj
-				.on('mousedown', function() {
+				.on('mousedown', function(event) {
 					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1) {
 						$obj.addClass('active');
 						$('.selected li.selected', $obj).removeClass('selected');
+
+						// Focus input only when not clicked on selected item.
+						if (!$(event.target).parents('.multiselect-list').length) {
+							$('input[type="text"]', $obj).focus();
+						}
 					}
 				})
 				.on('remove', function() {
@@ -428,6 +421,10 @@ jQuery(function($) {
 						popup_options = jQuery.extend(popup_options, getFilterPreselectField($obj, MS_ACTION_POPUP));
 					}
 
+					if (typeof popup_options['disable_selected'] !== 'undefined' && popup_options['disable_selected']) {
+						popup_options['disableids'] = Object.keys(ms.values.selected);
+					}
+
 					// Click used instead focus because in patternselect listen only click.
 					$('input[type="text"]', $obj).click();
 					return PopUp('popup.generic', popup_options, null, event.target);
@@ -487,11 +484,11 @@ jQuery(function($) {
 			})
 				.on('keyup', function(e) {
 					switch (e.which) {
-						case KEY.ARROW_DOWN:
-						case KEY.ARROW_LEFT:
-						case KEY.ARROW_RIGHT:
-						case KEY.ARROW_UP:
-						case KEY.ESCAPE:
+						case KEY_ARROW_DOWN:
+						case KEY_ARROW_LEFT:
+						case KEY_ARROW_RIGHT:
+						case KEY_ARROW_UP:
+						case KEY_ESCAPE:
 							return false;
 					}
 
@@ -565,13 +562,19 @@ jQuery(function($) {
 				})
 				.on('keydown', function(e) {
 					switch (e.which) {
-						case KEY.TAB:
-						case KEY.ESCAPE:
-							hideAvailable($obj);
+						case KEY_TAB:
+						case KEY_ESCAPE:
+							var $available = ms.values.available_div;
+
+							if ($available.parent().is(document.body)) {
+								hideAvailable($obj);
+								e.stopPropagation();
+							}
+
 							cleanSearchInput($obj);
 							break;
 
-						case KEY.ENTER:
+						case KEY_ENTER:
 							if ($input.val() !== '') {
 								var $selected = $('li.suggest-hover', ms.values.available_div);
 
@@ -584,7 +587,7 @@ jQuery(function($) {
 							}
 							break;
 
-						case KEY.ARROW_LEFT:
+						case KEY_ARROW_LEFT:
 							if ($input.val() === '') {
 								var $collection = $('.selected li', $obj);
 
@@ -600,7 +603,7 @@ jQuery(function($) {
 							}
 							break;
 
-						case KEY.ARROW_RIGHT:
+						case KEY_ARROW_RIGHT:
 							if ($input.val() === '') {
 								var $collection = $('.selected li', $obj);
 
@@ -616,13 +619,13 @@ jQuery(function($) {
 							}
 							break;
 
-						case KEY.ARROW_UP:
-						case KEY.ARROW_DOWN:
+						case KEY_ARROW_UP:
+						case KEY_ARROW_DOWN:
 							var $collection = $('li', ms.values.available_div.filter(':visible')),
 								$selected = $collection.filter('.suggest-hover').removeClass('suggest-hover');
 
 							if ($selected.length) {
-								$selected = (e.which == KEY.ARROW_UP)
+								$selected = (e.which == KEY_ARROW_UP)
 									? ($selected.is(':first-child') ? $collection.last() : $selected.prev())
 									: ($selected.is(':last-child') ? $collection.first() : $selected.next());
 
@@ -634,8 +637,8 @@ jQuery(function($) {
 
 							return false;
 
-						case KEY.BACKSPACE:
-						case KEY.DELETE:
+						case KEY_BACKSPACE:
+						case KEY_DELETE:
 							if ($input.val() === '') {
 								var $selected = $('.selected li.selected', $obj);
 
@@ -646,7 +649,7 @@ jQuery(function($) {
 									if (typeof item.disabled === 'undefined' || !item.disabled) {
 										var aria_text = sprintf(t('Removed, %1$s'), $selected.data('label'));
 
-										$selected = (e.which == KEY.BACKSPACE)
+										$selected = (e.which == KEY_BACKSPACE)
 											? ($selected.is(':first-child') ? $selected.next() : $selected.prev())
 											: ($selected.is(':last-child') ? $selected.prev() : $selected.next());
 
@@ -674,7 +677,7 @@ jQuery(function($) {
 										$aria_live.text(t('Can not be removed'));
 									}
 								}
-								else if (e.which == KEY.BACKSPACE) {
+								else if (e.which == KEY_BACKSPACE) {
 									/*
 									 * Pressing Backspace on empty input field should select last element in
 									 * multiselect. For next Backspace press to be able to remove it.
@@ -787,6 +790,9 @@ jQuery(function($) {
 					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1) {
 						$('.selected li.selected', $obj).removeClass('selected');
 						$(this).addClass('selected');
+
+						// preventScroll not work in IE.
+						$('input[type="text"]', $obj)[0].focus({preventScroll: true});
 					}
 				});
 
@@ -1184,7 +1190,7 @@ jQuery(function($) {
 	}
 
 	function isSearchFieldVisible($obj) {
-		return !$obj.hasClass('.search-disabled');
+		return !$obj.hasClass('search-disabled');
 	}
 
 	function getLimit($obj) {
